/*
 * Copyright 2012-2013 Canonical Ltd.
 *
 * This file is part of dialer-app.
 *
 * dialer-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * dialer-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.History 0.1
import Ubuntu.Telephony 0.1
import Ubuntu.Contacts 0.1
import "dateUtils.js" as DateUtils

Page {
    id: historyPage
    objectName: "historyPage"

    property string searchTerm
    property int delegateHeight: delegate.height
    property bool fullView: false
    property alias currentIndex: historyList.currentIndex
    property alias selectionMode: historyList.isInSelectionMode

    function activateCurrentIndex() {
        if (historyList.currentItem) {
            historyList.currentItem.activate();
        }
    }

    title: selectionMode ? i18n.tr("Select") : i18n.tr("Recent")
    anchors.fill: parent
    active: false

    Rectangle {
        anchors.fill: parent
        color: Theme.palette.normal.background
    }
    states: [
        PageHeadState {
            name: "select"
            when: selectionMode
            head: historyPage.head

            backAction: Action {
                objectName: "selectionModeCancelAction"
                iconName: "close"
                onTriggered: historyList.cancelSelection()
            }

            actions: [
                Action {
                    objectName: "selectionModeSelectAllAction"
                    iconName: "select"
                    onTriggered: {
                        if (historyList.selectedItems.count === historyList.count) {
                            historyList.clearSelection()
                        } else {
                            historyList.selectAll()
                        }
                    }
                },
                Action {
                    objectName: "selectionModeDeleteAction"
                    enabled: historyList.selectedItems.count > 0
                    iconName: "delete"
                    onTriggered: historyList.endSelection()
                }
            ]
        }
    ]

    onActiveChanged: {
        if (!active) {
            if (selectionMode) {
                historyList.cancelSelection();
            }
            historyList.resetSwipe()
            historyList.positionViewAtBeginning()
        }

    }

    // Use this delegate just to calculate the height
    HistoryDelegate {
        id: delegate
        visible: false
        property variant model: Item {
            property string senderId: "dummy"
            property variant participants: ["dummy"]
        }
    }

    // FIXME: this is a big hack to fix the placing of the listview items
    // when dragging the bottom edge
    flickable: null
    Connections {
        target: pageStack
        onDepthChanged: {
            if (pageStack.depth > 1)
                flickable = historyList
        }
    }

    HistoryEventModel {
        id: historyEventModel
        type: HistoryThreadModel.EventTypeVoice
        sort: HistorySort {
            sortField: "timestamp"
            sortOrder: HistorySort.DescendingOrder
        }
        filter: HistoryFilter {}
    }

    SortProxyModel {
        id: sortProxy
        sortRole: HistoryEventModel.TimestampRole
        sourceModel: historyEventModel
        ascending: false
    }

    MultipleSelectionListView {
        id: historyList
        objectName: "historyList"

        property var _currentSwipedItem: null

        function resetSwipe()
        {
            if (_currentSwipedItem) {
                _currentSwipedItem.resetSwipe()
                _currentSwipedItem = null
            }
        }

        function _updateSwipeState(item)
        {
            if (item.swipping) {
                return
            }

            if (item.swipeState !== "Normal") {
                if (_currentSwipedItem !== item) {
                    if (_currentSwipedItem) {
                        _currentSwipedItem.resetSwipe()
                    }
                    _currentSwipedItem = item
                }
            } else if (item.swipeState !== "Normal" && _currentSwipedItem === item) {
                _currentSwipedItem = null
            }
        }

        Connections {
            target: Qt.application
            onActiveChanged: {
                if (!Qt.application.active) {
                    historyList.currentIndex = -1
                }
            }
        }

        currentIndex: -1
        anchors.fill: parent
        listModel: sortProxy

        onSelectionDone: {
            for (var i=0; i < items.count; i++) {
                var event = items.get(i).model
                historyEventModel.removeEvent(event.accountId, event.threadId, event.eventId, event.type)
            }
        }
        onIsInSelectionModeChanged: {
            if (isInSelectionMode && _currentSwipedItem) {
                _currentSwipedItem.resetSwipe()
                _currentSwipedItem = null
            }
        }

        listDelegate: delegateComponent

        Component {
            id: delegateComponent
            HistoryDelegate {
                id: historyDelegate
                objectName: "historyDelegate" + index

                anchors{
                    left: parent.left
                    right: parent.right
                }

                selected: historyList.isSelected(historyDelegate)
                selectionMode: historyList.isInSelectionMode
                isFirst: model.index === 0
                locked: historyList.isInSelectionMode
                fullView: historyPage.fullView
                active: ListView.isCurrentItem

                // Animate item removal
                ListView.onRemove: SequentialAnimation {
                    PropertyAction {
                        target: historyDelegate
                        property: "ListView.delayRemove"
                        value: true
                    }

                    // reset swipe state
                    ScriptAction {
                        script: {
                            if (historyList._currentSwipedItem === historyDelegate) {
                                historyList._currentSwipedItem.resetSwipe()
                                historyList._currentSwipedItem = null
                            }

                            if (ListView.isCurrentItem) {
                                contactListView.currentIndex = -1
                            }
                        }
                    }

                    // animate the removal
                    UbuntuNumberAnimation {
                        target: historyDelegate
                        property: "height"
                        to: 1
                    }

                    PropertyAction {
                        target: historyDelegate
                        property: "ListView.delayRemove"
                        value: false
                    }
                }

                onItemPressAndHold: {
                    if (!historyList.isInSelectionMode) {
                        historyList.startSelection()
                    }
                    historyList.selectItem(historyDelegate)
                }

                onItemClicked: {
                    if (historyList.isInSelectionMode) {
                        if (!historyList.selectItem(historyDelegate)) {
                            historyList.deselectItem(historyDelegate)
                        }
                        return
                    }

                    historyDelegate.activate()
                }

                onSwippingChanged: historyList._updateSwipeState(historyDelegate)
                onSwipeStateChanged: historyList._updateSwipeState(historyDelegate)

                leftSideAction: Action {
                    iconName: "delete"
                    text: i18n.tr("Delete")
                    onTriggered:  historyEventModel.removeEvent(model.accountId, model.threadId, model.eventId, model.type)
                }
                property bool knownNumber: participants[0] != "x-ofono-private" && participants[0] != "x-ofono-unknown"
                rightSideActions: [
                    // FIXME: the first action should go to contac call log details page
                    Action {
                        iconName: unknownContact ? "contact-new" : "stock_contact"
                        text: i18n.tr("Details")
                        onTriggered: {
                            if (unknownContact) {
                                mainView.addNewPhone(phoneNumber)
                            } else {
                                mainView.viewContact(contactId)
                            }
                        }
                        visible: knownNumber
                        enabled: knownNumber
                    },
                    Action {
                        iconName: "message"
                        text: i18n.tr("Send message")
                        onTriggered: {
                            mainView.sendMessage(phoneNumber)
                        }
                        visible: knownNumber
                        enabled: knownNumber
                    }
                ]
            }
        }
    }

    Scrollbar {
        flickableItem: historyList
        align: Qt.AlignTrailing
    }
}
